/*Programa para el control de una vivienda con Arduino
Para ver las funciones con mas detallle consultar en el tomo principal */
#include <NECIRrcv.h>
#include <SoftwareSerial.h>
#include "Codigo_mando.h"
#include "pines.h"

NECIRrcv ir(IR) ;
SoftwareSerial BT(BT_RX,BT_TX);

void setup()
{
  Serial.begin(9600) ; //Inicializo la comunicacion serie con el PC
  ir.begin() ; //Inicio la recepcion del codigo IR
  BT.begin(9600); //Inicializo la comunicación Bluetooth

  pinMode(LED_GAS, OUTPUT); 
  pinMode(LED_LLAMA, OUTPUT); 
  pinMode(LED_MOVIMIENTO, OUTPUT);
  pinMode(LED_LLUVIA, OUTPUT); 

  pinMode(LED_V,OUTPUT);
  pinMode(LED_VLL,OUTPUT);
  pinMode(LUZ,OUTPUT);
  digitalWrite(LUZ,HIGH);

  pinMode(LED_PASILLO, OUTPUT);

  pinMode(ALARMA,OUTPUT);

  pinMode(TRIGGER,OUTPUT);
  pinMode(ECHO,INPUT);

  pinMode(PIR, INPUT);
  digitalWrite(PIR, LOW);

  Serial.print("Calibrando sensor ");
  for(int i = 0; i < 10; i++){
    Serial.print(".");
    delay(1000);
  }
  Serial.println("SENSOR LISTO");
  delay(50);
}

void loop()
{
  COMPROBAR_ESTADO();
  GAS();
  LLAMA();
  LLUVIA();
  MOV_ULTRA();
  MOV_PIR();
  REGULACION_VENTANAS();
  LEER_IR();
  LEER_BT();
}
/* Definimos las funciones utilizadas en el loop()*/
void COMPROBAR_ESTADO(){
  if(lectura_LLAMA()==0 && lectura_GAS()==0 && lectura_LLUVIA()==0) alarma_off();
}

//Funciones para la comprobacion de GAS
void GAS(){ 
  if (lectura_GAS()==1)
  {
    alarma();
    digitalWrite(LED_GAS,HIGH);
  }
  else
  {
    digitalWrite(LED_GAS,LOW);
  }
}

int lectura_GAS(){
  int valor_GAS=analogRead(SENSOR_GAS);
  Serial.println(valor_GAS);
  if(valor_GAS>maximo_GAS)
    return 1;
  else
    return 0;
}

//Funciones para la comprobacion de FUEGO
void LLAMA(){
  if (lectura_LLAMA()==1)
  {
    alarma();
    digitalWrite(LED_LLAMA,HIGH);
  }
  else
  {
    digitalWrite(LED_LLAMA,LOW);
  }
}

int lectura_LLAMA(){
  int valor_LLAMA=analogRead(SENSOR_LLAMA);
  Serial.println(valor_LLAMA);
  if(valor_LLAMA>maximo_LLAMA)
    return 1;
  else
    return 0;
}

//Funciones para la comprobacion de INUNDACION
void LLUVIA(){
  if (lectura_LLUVIA()==1)
  {
    alarma();
    digitalWrite(LED_LLUVIA,HIGH);
  }
  else
  {
    digitalWrite(LED_LLUVIA,LOW);
  }
}

int lectura_LLUVIA(){
  int valor_LLUVIA=analogRead(SENSOR_LLUVIA);
  Serial.println(valor_LLUVIA);
  if(valor_LLUVIA<maximo_LLUVIA)
    return 1;
  else
    return 0;
}

//Funcion para la regulacion de las VENTANAS
void REGULACION_VENTANAS(){
  int lectura_LDR=analogRead(LDR_V);
  Serial.println(lectura_LDR); //lectura analógica
  lectura_LDR = map(lectura_LDR, 0, 1023, 255, 0);
  analogWrite(LED_V, lectura_LDR);

  int lectura_LDR_2=analogRead(LDR_VLL);
  Serial.println(lectura_LDR_2); //lectura analógica
  lectura_LDR_2 = map(lectura_LDR_2, 0, 1023, 255, 0);
  analogWrite(LED_VLL, lectura_LDR_2);
}

//Funcion para deteccion de movimiento con ultrasonidos
void MOV_ULTRA(){
  digitalWrite(TRIGGER,LOW);
  delayMicroseconds(5);

  digitalWrite(TRIGGER,HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIGGER,LOW);

  float  distance=pulseIn(ECHO,HIGH);
  distance=distance*0.01657;
  Serial.println(distance);
  if(distance<maximo_distancia) 
    digitalWrite(LED_MOVIMIENTO,HIGH);
  else
    digitalWrite(LED_MOVIMIENTO,LOW);
}

//Funcion para sensor movimiento con PIR
void MOV_PIR(){
  if(digitalRead(PIR) == HIGH)
  {
    digitalWrite(LED_PASILLO, HIGH); 
  }
  else
    digitalWrite(LED_PASILLO, LOW); 
}

//Funciones para control alarma
void alarma(){
  tone(ALARMA, 2000,2000);
}
void alarma_off(){
  noTone(ALARMA);
}

//Funcion para leer IR
void LEER_IR(){
  unsigned long ircode ; //Variable que almacena el codigo leído por el receptor IR
  while (ir.available()) {  // Bucle que recibe datos de IR mientras esté disponible la conexión
    ircode = ir.read() ;
    Serial.print("Codigo:") ;
    Serial.println(ircode) ;
    switch (ircode) {
    case PLAY:
      digitalWrite(LUZ, HIGH);
      break;
    case STOP:
      digitalWrite(LUZ, LOW);
      break;
    case PAUSE:
      alarma_off();
      delay(10000);
      break;
    }
  }
}

//Funcion para recibir Bluetooth
void LEER_BT()
{
  char BTcode=BT.read();
  Serial.println(BTcode);
  switch (BTcode){
  case 'A':
    digitalWrite(LUZ, HIGH);
    break;
  case 'a':
    digitalWrite(LUZ, LOW);
    break;
  case 'B':
    alarma_off();
    delay(10000);
    break;
  }
}







